{#
Renders a ``<script>`` tag for raw JavaScript code.

:param js_code: raw JavaScript code to put in the tag.
:type js_code: str

#}
<script type="{{ type }}"{%if id %} id="{{ id }}"{% endif %}>
{{ js_code }}
</script>
