export const root: string
export const input: string
export const input_group: string
export const inline: string
export const spin_wrapper: string
export const spin_btn: string
export const spin_btn_up: string
export const spin_btn_down: string
export default ""
